-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Table with one item per key"
	interface: client

class P_DICTIONARY[G,K -> HASHABLE]

inherit
	P_TABLE[G,K]
		end

creation
	make

feature { P_DICTIONARY }
	
	container : P_HASH_SET[P_DICT_NODE[G,K]]

feature
	
	make,reset is
			-- Initialise.
		do
			!!container.make
		end

feature
	
	key_search(k : K) is
			-- Search key and set current item if successful.
			-- is_equal is used to compare keys.
		local
			node : P_DICT_NODE[G,K]
		do
			!!node
			node.set_key(k)
			container.search(node)
		end

	found : BOOLEAN is
			-- Result of key search.
		do
			Result := container.found
		end

	found_count : INTEGER is 1
			-- Number of items for found key (always 1 for dictionary).

feature -- from ANY
	
	copy(other : like Current) is
			-- Copy.
		do
			container.copy(other.container)
		end

	is_equal(other : like Current) : BOOLEAN is
			-- Equality.
		do
			Result := container.is_equal(other.container)
		end

feature -- from P_CONTAINER

	count : INTEGER is
			-- Number of items/keys.
		do
			Result := container.count
		end
	
	item : G is
			-- Current item.
		local
			node : P_DICT_NODE[G,K]
		do
			node := container.item
			Result := node.item
		end

	key : K is
			-- Current key.
		local
			node : P_DICT_NODE[G,K]
		do
			node := container.item
			Result := node.key
		end

feature

	is_writable : BOOLEAN is true
		
	is_key_writable(k : K) : BOOLEAN is
			-- Is the key already used?
		local
			node : P_DICT_NODE[G,K]
		do
			!!node
			node.set_key(k)
			Result := not container.has(node)
		end

feature
	
	add(it : G; k : K) is
			-- Add item and corresponding key.
			-- (key must be available).
		local
			node : P_DICT_NODE[G,K]
		do
			!!node
			node.set_item(it)
			node.set_key(k)
			container.add(node)
		end

	replace(it : G) is
			-- Replace current item (key unchanged).
		do
			container.item.set_item(it)
		end
	
	remove is
			-- Remove current item and associated key.
		do
			container.remove
		end

feature -- Collection interface

	to_item_list : P_LIST[G] is
			-- List of items.
		local
			rlist : P_LINKED_LIST[G]
			it : P_ITERATOR[P_DICT_NODE[G,K]]
		do
			!!rlist.make
			Result := rlist

			from
				it := container.to_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				Result.add(it.item.item)
				it.forth
			end
		ensure then
			done: Result.count = count
		end

	to_key_list : P_LIST[K] is
			-- List of keys.
		local
			rlist : P_LINKED_LIST[K]
			it : P_ITERATOR[P_DICT_NODE[G,K]]
		do
			!!rlist.make
			Result := rlist
			
			from
				it := container.to_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				Result.add(it.item.key)
				it.forth
			end
		ensure then
			done: Result.count = count
		end

end
