-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Misc
	description: "String/Basic types conversions"
	interface: client, mixin

class P_FORMAT

feature { NONE }
	
	Fractional_precision : INTEGER is 8 -- 10 ^ Fractional_precision < (2 ^ Integer_bits) - 1

	Max_digits : INTEGER is 100 --? better
	
	Lower_case : STRING is "abcdefghijklmnopqrstuvwxyz"
	Upper_case : STRING is "ABCDEFGHIJKLMNOPQRSTUVWXYZ"

feature -- Misc character (character set independent)

	is_hexdigit(ch : CHARACTER) : BOOLEAN is
			-- Is this character hexadecimal? [0-9a-fA-F]
		do
			Result := is_digit(ch) or has_character("abcdefABCDEF",ch)
		end

	is_digit(ch : CHARACTER) : BOOLEAN is
			-- Is this character a digit? [0-9]
		do
			Result := ch = '0' or else ch = '1' or else ch = '2' or else ch = '3' 
						or else ch = '4' or else ch = '5' or else ch = '6' 
						or else ch = '7' or else ch = '8' or else ch = '9'
		end

	is_alpha(ch : CHARACTER) : BOOLEAN is
			-- Is this character an ASCII alphabetic character? [a-zA-Z]
		do
			Result := has_character(Lower_case,ch) or has_character(Upper_case,ch)
		end

	is_lower(ch : CHARACTER) : BOOLEAN is
			-- Is this character [a-z]?
		do
			Result := has_character(Lower_case,ch)
		end

	is_upper(ch : CHARACTER) : BOOLEAN is
			-- Is this character [A-Z]?
		do
			Result := has_character(Upper_case,ch)
		end

feature -- CHARACTER -- ascii case

	to_lower(ch : CHARACTER) : CHARACTER is
			-- Convert ASCII character to lower case (strictly a-z).
		local
			i : INTEGER
		do
			Result := ch
			i := index_of(Upper_case,ch) 
			if i > 0 then
				Result := Lower_case.item(i)
			end
		ensure
			done: not is_upper(Result)
			down: is_upper(ch) implies is_lower(Result)
			keep: (not is_upper(ch)) implies ch = Result
		end

	to_upper(ch : CHARACTER) : CHARACTER is
			-- Convert ASCII character to upper case (stricly A-Z).
		local
			i : INTEGER
		do
			-- slower but portable method 
			-- (compared to character math)
			Result := ch
			i := index_of(Lower_case,ch)
			if i > 0 then
				Result := Upper_case.item(i)
 			end
		ensure
			done: not is_lower(Result)
			up: is_lower(ch) implies is_upper(Result)
			keep: (not is_lower(ch)) implies ch = Result
		end

feature -- CHARACTER/INTEGER
	
	code_to_char(code : INTEGER) : CHARACTER is
			-- Convert character code (integer) to CHARACTER object.
		local
			ps : P_SYSTEM
		do
			!!ps
			Result := ps.code_to_char(code)
		ensure
			done: Result.code = code
		end

feature -- INTEGER/STRING
	
	int_to_str(int : INTEGER) : STRING is
			-- Convert integer to string.
			-- Format: "[-](digits)+"
		do
			Result := int_to_str_min(int,0)
		ensure
			is_copy: Result /= Void
		end
	

	int_to_str_min(int : INTEGER; min : INTEGER) : STRING is
			-- Convert integer to string, 
			-- fill with 0 to go up to 'min' digits.
		local
			i, outdigit : INTEGER
		do
			!!Result.make(0)
			
			from
				if int < 0 then
					i := - int
					--? include sign: outdigit := 1 
				else 
					i := int 
				end
			variant
				min.max(Max_digits) + 1 - outdigit
			until
				i = 0 and outdigit >= min 
			loop
				Result.append_character(int_to_char(i \\ 10))
				i := i // 10
				outdigit := outdigit + 1 
			end

			if int = 0 then
				Result.append_character(int_to_char(0))
			end

			if int < 0 then
				Result.append_character('-')
			end
		
			invert(Result)
		ensure
			min: Result.count >= min
			is_copy: Result /= Void
		end

	int_to_hex(int : INTEGER) : STRING is
			-- Convert integer to string, hexadecimal format.
		do
			Result := int_to_hex_min(int,0)
		ensure
			is_copy: Result /= Void
		end

	int_to_hex_min(int : INTEGER; min : INTEGER) : STRING is
			-- Convert integer to string, hexadecimal format.
			-- fill with 0 to go up to 'min' digits.
		local
			i, outdigit : INTEGER
		do
			!!Result.make(0)
			
			from
				if int < 0 then i := - int else i := int end
			variant
				min.max(Max_digits) + 1 - outdigit
			until
				i = 0 and outdigit >= min 
			loop
				Result.append_character(hex_to_char(i \\ 16))
				i := i // 16
				outdigit := outdigit + 1 
			end

			if int < 0 then
				Result.append_character('-')
			end
		
			invert(Result)
		ensure
			min: Result.count >= min
			is_copy: Result /= Void
		end
	
	str_to_int(str : STRING) : INTEGER is
			-- Convert string to integer.
			-- Format: "[+|-](digits)*"
		require
			valid: str /= Void
		local
			i : INTEGER;
			negative : BOOLEAN
		do
			if str.count > 0 then
				i := 1
				
				-- get sign
				check first: i = 1 end
				if str.item(1) = '-' then
					negative := true
					i := i + 1
				elseif str.item(1) = '+' then
					i := i + 1
				end

				-- get digits
				from
				variant
					str.count - i + 1
				until
					i > str.count or else (not is_digit(str.item(i))) 
				loop
					Result := 10 * Result + char_to_int(str.item(i))
					i := i + 1
				end

				if negative then
					Result := (- Result)
				end
			end
		end

feature -- REAL/STRING, DOUBLE/STRING
	
	str_to_double(str : STRING) : DOUBLE is
			-- String to DOUBLE.
			-- Format: "[+|-](digits)*[.[(digits)*]][{e|E}[{+|-}](digits)*]"
		require
			valid: str /= Void
		local
			frac_fact, tmp : DOUBLE
			exp,i : INTEGER
			negative : BOOLEAN
		do
			if str.count > 0 then
				i := 1

				-- get sign
				check first: i = 1 end
				if str.item(1) = '-' then
					negative := true
					i := i + 1
				elseif str.item(1) = '+' then
					i := i + 1
				end

				-- get integer part
				from
				variant
					str.count - i + 1
				until
					i > str.count or else (not is_digit(str.item(i))) 
				loop
					Result := (Result * 10.0) + char_to_int(str.item(i))
					i := i + 1
				end

				-- get fractional part
				if i < str.count and then str.item(i) = '.' then
					from
						i := i + 1
						frac_fact := 0.1
					variant
						str.count + 1 - i
					until
						i > str.count or else not is_digit(str.item(i))
					loop
						Result := Result + (frac_fact * char_to_int(str.item(i)))
						frac_fact := frac_fact / 10
						i := i + 1
					end
				end

				-- get exponent
				if i < str.count and then (str.item(i) = 'e' or str.item(i) = 'E') then
					i := i + 1
					exp := str_to_int(str.substring(i,str.count))
					if exp >= 0 then
						Result := Result * (10.0 ^ exp)
					else
						-- avoid negative power (not portable)  
						-- Result := Result * (1.0 / (10.0 ^ -exp))
						tmp := 1.0
						tmp := tmp / (10.0 ^ -exp)
						Result := Result * tmp
					end
				end

				-- insert sign
				if negative then
					Result := (- Result)
				end
			end -- str.count
		end

	real_to_str(r : REAL) : STRING is
			-- Convert REAL to string. 
			-- Format: "[-]<frac-int>E<exp-int>" 
		do
			Result := double_to_str(r)
		ensure
			is_copy: Result /= Void
		end

	str_to_real(str : STRING) : REAL is
			-- Convert string to REAL.
		require
			valid: str /= Void
		local
			sys : P_SYSTEM
		do
			!!sys
			Result := sys.double_to_real(str_to_double(str))
		end

	double_to_str(r : DOUBLE) : STRING is
			-- Convert DOUBLE to string.
		local
			rl : DOUBLE;
			int,prec,exp : INTEGER;
			ps : P_SYSTEM
		do
			!!ps

			!!Result.make(0)

			-- sign
			if r < 0 then
				Result.append_character('-')
				rl := - r
			else
				rl := r
			end
			check positive: rl >= 0 end

			-- exponent
			from until rl < Max_display_integer loop
				exp := exp + 1
				rl := rl / 10.0
			end
			from until rl > Min_display_integer loop
				exp := exp - 1
				rl := rl * 10.0
			end
			check rl_in: rl < Max_display_integer and rl > Min_display_integer end

			-- integer
			int := ps.floor(rl)
			Result.append_string(int_to_str(int))
			rl := rl - int
			
			-- fractional part
			if rl /= 0.0 then
				Result.append_character('.') 

				from
				invariant
					inf_one: rl < 1
					precision: prec < 25
				until
					rl = 0 or prec > 10 --? better precision
				loop
					rl := rl * 10.0
					int := ps.floor(rl)
					Result.append_character(int_to_char(int))
					rl := rl - int
					prec := prec + 1
				end
			end

			-- actually add exponent
			if exp /= 0 then
				Result.append_character('e')
				Result.append_string(int_to_str(exp))
			end
		ensure
			is_copy: Result /= Void
		end

feature { NONE } -- double_to_str constants

	Max_display_integer : DOUBLE is 1000000.0
	Min_display_integer : DOUBLE is 0.1

feature -- CHARACTER/INTEGER

	char_to_int(ch : CHARACTER) : INTEGER is
			-- Convert digit character (0-9) to INTEGER.
		require
			valid: -- ch should be '0'/'9', otherwise Result = 0
		do
			Result := char_to_hex(ch)
			if Result >= 10 then
				Result := 0
			end
		ensure
			done: Result >= 0 and Result < 10
		end
	
	int_to_char(int : INTEGER) : CHARACTER is
			-- Convert 0-9 integer to character.
		require
			valid: int >= 0 and int < 10
		do
			inspect int -- bulky but charset independent
			when 0 then Result := '0'
			when 1 then Result := '1'
			when 2 then Result := '2'
			when 3 then Result := '3'
			when 4 then Result := '4'
			when 5 then Result := '5'
			when 6 then Result := '6'
			when 7 then Result := '7'
			when 8 then Result := '8'
			when 9 then Result := '9'
			end
		ensure
			done: char_to_int(Result) = int
		end

feature -- hexadecimal CHARACTER/INTEGER

	hex_to_char(int : INTEGER) : CHARACTER is
			-- Convert hex digit to char.
		require
			hex: int >= 0 and int < 16
		do
			inspect int -- bulky but charset independent.
			when 0 then Result := '0'
			when 1 then Result := '1'
			when 2 then Result := '2'
			when 3 then Result := '3'
			when 4 then Result := '4'
			when 5 then Result := '5'
			when 6 then Result := '6'
			when 7 then Result := '7'
			when 8 then Result := '8'
			when 9 then Result := '9'
			when 10 then Result := 'a'
			when 11 then Result := 'b'
			when 12 then Result := 'c'
			when 13 then Result := 'd'
			when 14 then Result := 'e'
			when 15 then Result := 'f'
			end
		ensure
			done: char_to_hex(Result) = int
		end

	char_to_hex(ch : CHARACTER) : INTEGER is
			-- Convert hex char to integer.
		require
			hex: -- ch 0-9A-Za-z
		do
			inspect ch -- bulky but charset independent.
			when '0' then Result := 0
			when '1' then Result := 1
			when '2' then Result := 2
			when '3' then Result := 3
			when '4' then Result := 4
			when '5' then Result := 5
			when '6' then Result := 6
			when '7' then Result := 7
			when '8' then Result := 8
			when '9' then Result := 9
			when 'a','A' then Result := 10
			when 'b','B' then Result := 11
			when 'c','C' then Result := 12
			when 'd','D' then Result := 13
			when 'e','E' then Result := 14
			when 'f','F' then Result := 15
			else -- nop
			end
		ensure
			done: Result >= 0 and Result < 16
		end

feature -- Strings & characters
	
	index_of(str : STRING; ch : CHARACTER) : INTEGER is
			-- Index of first occurence of 'ch' in 'str'.
			-- Time: O(str.count)
		require
			valid: str /= Void
		local
			i : INTEGER
		do
			from
				i := 1
			variant
				str.count + 1 - i
			until
				i > str.count or Result > 0
			loop
				if str.item(i) = ch then
					Result := i
				end
				i := i + 1
			end
		ensure
			done: (not has_character(str,ch)) = (Result = 0)
			found: Result > 0 implies str.item(Result) = ch
		end

	index_of_last(str : STRING; ch : CHARACTER) : INTEGER is
			-- Index of last occurence of 'ch' in 'str'.
			-- Time: O(str.count)
		require
			valid: str /= Void
		local
			i : INTEGER
		do
			from
				i := str.count
			variant
				i
			until
				i = 0 or Result > 0
			loop
				if str.item(i) = ch then
					Result := i
				end
				i := i - 1
			end
		ensure
			done: (not has_character(str,ch)) = (Result = 0)
			found: Result > 0 implies str.item(Result) = ch
		end

	has_character(str : STRING; ch : CHARACTER) : BOOLEAN is
			-- Does 'str' has 'ch'?
			-- Time: O(str.count)
		require
			valid: str /= Void
		do
			Result := index_of(str,ch) > 0
		end
		
feature -- String utilities

	tokenize(input : STRING; separators : STRING) : P_LIST[STRING] is
			-- Basic tokeniser for a string, returns a list of tokens 
			-- made of chars which are not in 'separators'.
		require
			valid_input: input /= Void
			valid_separator: separators /= Void
		local
			curtok : STRING;
			i : INTEGER
		do
			!P_LINKED_LIST[STRING]!Result.make

			!!curtok.make(0)
			from
				i := 1
			invariant
				token: curtok /= Void
			variant
				input.count + 1 - i
			until
				i > input.count
			loop
				if has_character(separators,input.item(i)) then
					if not curtok.empty then
						Result.add(curtok)
						!!curtok.make(0)
					end
				else
					curtok.append_character(input.item(i))
				end
				i := i + 1
			end
			
			if not curtok.empty then
				Result.add(curtok)
			end
		end

	start_case_equal(a,b : STRING) : BOOLEAN is
			-- Test case-independently if two strings have equal beginnings.
		require
			valid: a /= Void and b /= Void
		local
			len : INTEGER
		do
			Result := true
			len := (a.count).min(b.count)
			if len > 0 then
				Result := case_equal(a.substring(1,len),b.substring(1,len))
			end
		end

	start_equal(a,b : STRING) : BOOLEAN is
			-- Test if two strings have equal beginnings.
		require
			valid: a /= Void and b /= Void
		local
			len : INTEGER
		do
			len := (a.count).min(b.count)
			if len > 0 then
				Result := (a.substring(1,len)).is_equal(b.substring(1,len))
			elseif len = 0 and a.count = b.count then
				Result := true --? really? good policy?
			end
		end
 
	
	case_equal(a,b : STRING) : BOOLEAN is
			-- Test case-independently if two strings are equal.
		require
			valid: a /= Void and b /= Void
		local
			al,bl : STRING
		do
			!!al.make(0)
			al.copy(a)
			al.to_lower
			!!bl.make(0)
			bl.copy(b)
			bl.to_lower
			Result := al.is_equal(bl)
		end

	invert(str : STRING) is
			-- Invert string.
			-- Time: O(str.count)
		require
			valid: str /= Void
		local
			i, end_count : INTEGER;
			ch : CHARACTER
		do
			from
				i := 1
				end_count := str.count // 2
			variant
				end_count + 1 - i
			until
				i > end_count
			loop
				ch := str.item(i)
				str.put(str.item(str.count + 1 - i),i)
				str.put(ch,str.count + 1 - i)

				i := i + 1
			end
		ensure
			done: -- invert(str) = old str
		end

end
