-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Abstract linked structure"
	interface: inheritance

class P_LINKED_LIST[G]

inherit
	P_LIST[G]
		end

creation
	make

feature
	
	make, reset is
		do
			first_node := Void
			last_node := Void
			current_node := Void
			count := 0
			make_traversable
		end

feature { P_MUTATOR } -- More attributes
	
	current_node : P_LINKED_NODE[G]
	
	first_node : P_LINKED_NODE[G]
		
	last_node: P_LINKED_NODE[G]

feature
	
	count : INTEGER;
			-- Number of elements in the data structure.

feature { P_MUTATOR }
	
	set_current_node(cnode : P_LINKED_NODE[G]) is
		require
			valid: cnode /= Void
		do
			current_node := cnode
		end

feature -- Iteration

	iterator : P_ITERATOR[G] is
			-- Iterator on this list.
		local
			actual : P_LINKED_ITERATOR[G]
		do
			!!actual.make(Current)
			Result := actual
		end

feature -- Lock status

	is_locked : BOOLEAN is
			-- Is current item locked?
			-- Time: O(1)
		do
			Result := current_node.is_locked
		end

feature -- Access
	
	item : G is
			-- Current item.
			-- Time: O(1)
		do
			Result := current_node.item
		end

feature -- Action

	add(it : G) is
			-- Add an item to the end of the list.
			-- Time: O(1)
		local
			node : P_LINKED_NODE[G]
		do
			!!node
			node.set_item(it)
			node.set_next(Void)
			
			if first_node = Void then
				node.set_previous(Void)
				first_node := node 
			else
				node.set_previous(last_node)
				last_node.set_next(node)
			end
			last_node := node
			current_node := node

			count := count + 1
		end

	replace(it: G) is
			-- Replace the current item.
			-- Time: O(1)
		do
			current_node.set_item(it)
		end

	insert(it : G) is
			-- Insert an item before the current item.
			-- Time: O(1)
		local
			node : P_LINKED_NODE[G]
		do
			!!node
			node.set_previous(current_node.previous)
			node.set_item(it)
			node.set_next(current_node)
			if current_node.is_first then
				first_node := node
			else
				current_node.previous.set_next(node)
			end
			current_node.set_previous(node)
			current_node := node

			count := count + 1
		end

	remove is
			-- Remove the current item.
			-- Time: O(1)
		do
			if not current_node.is_first then
				current_node.previous.set_next(current_node.next)
			else
				first_node := current_node.next
			end
			if not current_node.is_last then
				current_node.next.set_previous(current_node.previous)
				current_node := current_node.next
			else
				last_node := current_node.previous
				current_node := last_node
			end

			count := count - 1
		end

feature -- Sorting
	
	sort is
			-- Insertion sort.
			-- Time: O(count^2/4) comparisons; O(count) swaps
		local
			ext_loop,int_loop : P_LINKED_NODE[G];
			citem : G
		do
			--? better algorithm / use iterators and move to P_LIST
			
			-- Iterate 2 .. N and place the external loop current item at its 
			-- place in the previous items which are known to be sorted.

			if not is_empty then
				from
					ext_loop := first_node
					ext_loop := ext_loop.next
				invariant
					sorted: -- start -> ext_loop is sorted
				until
					ext_loop = Void
				loop
					-- go back if current item not already bigger that the last of the sorted one.
					if not order_relation.ordered(ext_loop.previous.item,ext_loop.item) then
						from -- internal loop: go back to insertion point
							int_loop := ext_loop
							citem := ext_loop.item
							-- remove current ext_loop ...
							set_current_node(ext_loop)
							remove
						invariant
							valid_ext: ext_loop /= Void
							valid_previous: ext_loop.previous /= Void
						until
							int_loop = Void
						loop
							int_loop := int_loop.previous

							if int_loop = Void then
								-- insert at the head of the list
								current_node := first_node
								insert(citem)
							elseif order_relation.ordered(int_loop.item, citem) then 
								-- insert after int_loop
								if int_loop.is_last then
									add(citem)
								else
									set_current_node(int_loop.next)
									insert(citem)
								end

								-- stop iteration
								int_loop := Void
							end
						end -- end internal loop
					end
					
					-- forth
					ext_loop := ext_loop.next
				end -- end external loop

				-- reset position to first
				current_node := first_node 
			end
		end

invariant
	sym_current_first: (current_node = Void) = (first_node = Void)
	sym_first_last: (first_node = Void) = (last_node = Void)
	correct_bounds: (not is_empty) implies (first_node.previous = Void and last_node.next = Void)

end
