-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Iteration
	description: "Abstract one-way linear iterator"
	interface: abstract, client

deferred class P_ONEWAY_ITERATOR[G]

inherit
	P_MUTATOR
		redefine
			copy
		end

feature -- Copy

	copy(other : like Current) is
			-- Cannot copy iterator.
		do
			check not_allowed: false end
		end

feature -- Access

	outside : BOOLEAN is
			-- Is the iterator outside the container?
		deferred
		end

	iterable : BOOLEAN is
			-- Is the associated structure iterable (not empty)?
		deferred
		end

	item : G is
			-- Return the item at the current position.
		require
			inside: not outside
		deferred
		end

feature -- Synchronization
	
	synchronize_container is
			-- Set the current item of the target container to be 
			-- the one currently pointed at by the iterator.
			-- THIS METHOD DOES CHANGE THE CONTAINER STATE.
		require
			inside: not outside
		deferred
		ensure
			done: -- container current position changed
		end
					
feature -- Iteration
	
	first is
			-- Go to first item of the container.
		require
			outside: outside
		deferred
		ensure
			done: iterable implies (not outside)
		end

	synchronize is
			-- Set the iterator so that the iterator current item is 
			-- the current item of the container. Do not change the 
			-- container. 
		require
			outside: outside
			iterable: iterable
		deferred
		ensure
			done: not outside
		end

	forth is
			-- Next item.
		require
			inside: not outside
		deferred
		end

	stop is
			-- Finish.
		require
			inside: not outside
		deferred
		ensure
			done: outside
		end

feature -- Iterator locking
	
	is_locked : BOOLEAN is
			-- Is the current iterator locked?
		deferred
		end
	
	lock is
			-- Lock iterator: lock the current item into the container 
			-- and leave the iterator.
		require
			inside: not outside
			unlocked: not is_locked
		deferred
		ensure
			locked: is_locked
			outside: outside
		end

	unlock is
			-- Unlock iterator
		require
			locked: is_locked
			outside: outside
		deferred
		ensure
			inside: not outside
			not_locked: not is_locked
		end
			
feature -- Variant support

	forward_variant : INTEGER is
			-- Forward variant for forth-only loops.
		deferred
		end

invariant
	outside_locked: (not outside) implies (not is_locked)

end
