-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "One-way traversable linear container"
	interface: abstract

deferred class P_ONEWAY_TRAVERSABLE[G]
	
inherit
	P_CONTAINER[G]
		end

feature { NONE } -- Attributes

	protcount : INTEGER -- Number of active iterators.
	lockcount : INTEGER -- Number of locked iterators.

feature { NONE } -- Creation
	
	make_traversable is
			-- Called by heirs' creators.
		do
			protcount := 0
			lockcount := 0
		ensure
			done: not is_protected
		end

feature -- Iteration

	iterator : P_ONEWAY_ITERATOR[G] is
			-- Get a linear iterator on this container.
		deferred
		ensure
			done: Result /= Void
			is_shallow_copy: -- Result /= previous Result
		end

	is_protected : BOOLEAN is
			-- Is an iterator inside the data structure?
		do
			Result := (protcount /= 0)
		end

feature -- Lock status

	has_locked : BOOLEAN is
			-- Does this container has one or more locked items?
		do
			Result := (lockcount /= 0)
		end

	
	is_locked : BOOLEAN is
			-- Is the current item locked?
		require
			meaningful: not is_empty
		deferred
		ensure
			has_locked: Result = true implies has_locked
		end

feature { P_MUTATOR } -- Lock management

	add_lock is
		do
			lockcount := lockcount + 1
		end

	remove_lock is
		do
			lockcount := lockcount - 1
		end

feature { P_MUTATOR } -- Protection management

	protect is
		do
			protcount := protcount + 1
		end

	unprotect is
		do
			protcount := protcount - 1
		ensure
			valid_protection: protcount >= 0
		end

feature -- General (from P_CONTAINER)
	
	is_writable : BOOLEAN is
			-- Is the structure writable?
		do
			Result := not is_protected
		end

invariant
	valid_protect_count: protcount >= 0
	protection: is_protected implies (not is_writable)

end
