-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Abstract table"
	interface: abstract

deferred class P_TABLE[G,K]

inherit
	P_CONTAINER[G]
		end
	P_SEARCHABLE[G,K]
		rename
			search as key_search
		redefine
			key_search
		end

feature -- Search

	key_search(k : K) is
			-- Search by key (using is_equal to compare keys).
		deferred
		end

feature -- Status

	key : K is
			-- Current key.
		require
			readable: is_readable
			not_empty: not is_empty
		deferred
		end
	
	is_key_writable(k : K) : BOOLEAN is
			-- Can data be added with this key?
			-- (e.g., support for Dictionaries)
		require
			valid: not is_void(k)
		deferred
		end

feature -- Update

	add(it : G; k : K) is
			-- Add a new item (with its key).
		require
			writable: is_writable
			not_void_key: not is_void(k)
			acceptable_key: is_key_writable(k)
		deferred
		ensure
			keep_reference: -- keep copy of item and key.
		end

	replace(it : G) is
			-- Replace current item (keep the current key).
		require
			meaningful: not is_empty
			writable: is_writable
		deferred
		ensure
			keep_reference: -- keep copy of item.
		end

	remove is
			-- Remove the current item (and its corresponding key
			-- if it is associated with no other items).
		require
			meaningful: not is_empty
			writable: is_writable
		deferred
		end

feature -- Conversion

	to_key_list : P_LIST[K] is
			-- Return a list of keys in the table.
		deferred
		ensure
			valid: Result /= Void
			is_shallow_copy: -- new list generated each time (but real keys)
		end

	to_item_list : P_LIST[G] is
			-- Return a list of items in the table.
		deferred
		ensure
			valid: Result /= Void
			is_shallow_copy: -- new list generated each time (but real items)
		end

end

