-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Test
	description: "Substring for toy compress"
	interface: client

class TC_SUBSTRING

inherit
	HASHABLE
		redefine
			is_equal,copy
		end

creation
	make

feature { NONE } -- Creation
	
	make(src : STRING; offs : INTEGER; len : INTEGER) is
			-- Create substring from (constant) string.
		require
			valid_src: src /= Void and not src.empty
			valid_offset: offs > 0
			valid_lenght: len > 0
			low_bound: offs <= src.count
			up_bound: offs + len - 1 <= src.count
		local
			i : INTEGER
		do
			-- attrib
			master := src
			offset := offs
			lenght := len
			
			-- calculate hash value
			from
				i := count
			variant i until i = 0 loop
				hash := hash + item(i).code --? better
				i := i - 1
			end
		ensure
			keep_reference: master = src -- and must remain constant
			done: lenght = len and offset = offs
		end
	
feature -- Attributes

	master : STRING
	offset,lenght : INTEGER

	hash : INTEGER -- cache for hash value

feature -- Read-only string

	count : INTEGER is
		do
			Result := lenght
		end

	item(index : INTEGER) : CHARACTER is
		require
			valid_index: index > 0 and index <= count
		do
			Result := master.item(offset + index - 1)
		end

feature -- GENERAL/HASHABLE

	is_equal(other : like Current) : BOOLEAN is
		local
			i : INTEGER
		do
			if count = other.count then
				from
					Result := true
					i := 1
				variant count + 1 - i until 
					Result = false or i > count 
				loop
					Result := item(i) = other.item(i)
					i := i + 1
				end
			end
		end

	copy(other : like Current) is
		do
			master := other.master
			offset := other.offset
			lenght := other.lenght
			hash := other.hash
		end

	hash_code : INTEGER is
		do
			Result := hash
		end

end -- class TC_SUBSTRING
