-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 
-- Pylon Test

indexing
	cluster: Pylon, Test
	description: "Testing linear containers (lists and dispensers)"
	interface: client

class TEST_COLLECTION

inherit
	TESTER
		end

creation
	execute

feature { NONE }
	
	int_list : P_LIST[INTEGER]
	int_disp : P_DISPENSER[INTEGER]

	Size : INTEGER is 20

feature 
	
	execute is
			-- Creation/driver.
		do
			put_title("LINKED LIST:")
			!P_LINKED_LIST[INTEGER]!int_list.make
			test_integer_list
			
			put_title("INDEXED LIST:")
			!P_SEQUENCE_LIST[INTEGER]!int_list.make
			test_integer_list
		
			put_title("CONVERSION TEST:")
			test_conversion

			put_title("STACK:")
			!P_STACK[INTEGER]!int_disp.make
			test_integer_dispenser

			put_title("QUEUE:")
			!P_QUEUE[INTEGER]!int_disp.make
			test_integer_dispenser
		end

	test_conversion is
			-- Convert list from array to linked.
		local
			other : P_LIST[INTEGER]
		do
			!P_SEQUENCE_LIST[INTEGER]!int_list.make
			fill_ordered
			print_collection

			!P_LINKED_LIST[INTEGER]!other.make
			other.from_list(int_list)
			int_list := other
			print_collection

			put_nl
		end

	test_integer_dispenser is
			-- Test an integer dispenser.
		do
			push(1)
			push(2)
			push(3)
			pop
			push(4)
			push(5)
			pop
			pop
			pop
			push(6)
			pop
			pop
			put_nl
		ensure
			done: int_disp.is_empty
		end

	test_integer_list is
			-- Various test on an integer list.
		do
			put_title("singleton:")
			fill_singleton
			update_start

			put_title("* simple list:")
			fill_simple
			print_collection
			update_start
			update_end
			sort_list

			put_title("* ordered:")
			fill_ordered
			print_collection
			reverse_print
			update

			put_title("* reverse ordered:")
			fill_reverse_ordered
			print_collection
			reverse_print

			put_title("* constant:")
			fill_constant
			print_collection

			put_title("* remove all.")
			remove_all
		end

feature { NONE } -- Test dispenser

	push(i : INTEGER) is
			-- Dispenser: push
		require
			has_disp: int_disp /= Void
		do
			put_string(">")
			put_string(i.out)
			put_string(" ")
			int_disp.add(i)
		end

	pop is
			-- Dispenser: pop
		require
			has_disp: int_disp /= Void
		do
			put_string("<")
			put_string(int_disp.item.out)
			put_string(" ")
			int_disp.remove
		end


feature { NONE } -- Init list
			
	fill_singleton is
			-- Make a single item list.
		require
			has_list: int_list /= Void
		do
			int_list.reset
			int_list.add(Size)
		ensure
			done: int_list.count = 1
		end

	fill_simple is
			-- Fill list with a few numbers.
		require
			has_list: int_list /= Void
		do
			int_list.reset
			int_list.add(31)
			int_list.add(12)
			int_list.add(117)
			int_list.add(20)
		end

	fill_ordered is
			-- Fill int list in order.
		require
			has_list: int_list /= Void
		local
			int : INTEGER
		do
			from
				int_list.reset
				int := 1
			variant
				Size - int + 1
			until
				int > Size
			loop
				int_list.add(int)
				int := int + 1
			end
		end

	fill_reverse_ordered is
			-- Fill int_list in order.
		require
			has_list: int_list /= Void
		local
			int : INTEGER
		do
			from
				int_list.reset
				int := Size
			variant
				int
			until
				int < 1
			loop
				int_list.add(int)
				int := int - 1
			end
		end

	fill_constant is
			-- Fill int list with a single number.
		require
			has_list: int_list /= Void
		local
			int : INTEGER
		do
			from
				int_list.reset
				int := 1
			variant
				Size - int + 1
			until
				int > Size
			loop
				int_list.add(Size)
				int := int + 1
			end
		end

	remove_all is
			-- Remove all items one by one.
		require
			has_list: int_list /= Void
		do
			from
			variant int_list.count until 
				int_list.is_empty 
			loop
				int_list.remove
			end
		ensure
			done: int_list.is_empty
		end

feature { NONE } -- List test helper

	goto_start(it : P_ITERATOR[INTEGER]) is
			-- Position on first item.
		require
			valid_list: not int_list.is_empty
		do
			it.first
			it.synchronize_container
			it.stop
		end

	goto_end(it : P_ITERATOR[INTEGER]) is
			-- Position on last item.
		require
			valid_list: not int_list.is_empty
		do
			it.last
			it.synchronize_container
			it.stop
		end

feature -- List test
	
	update is
			-- Test access (insert/replace/remove)
		local 
			it : P_ITERATOR[INTEGER];
			step : INTEGER
		do
			check valid_list: not int_list.is_empty end
			
			step := int_list.count // 4

			put_string("replacement at #")
			put_string(step.out)
			simple_forward_move(step)
			int_list.replace(100)

			put_string(", insertion at #")
			put_string((2*step).out)
			simple_forward_move(2 * step)
			int_list.insert(101)

			-- go to end
			it := int_list.iterator
			it.last
			it.synchronize_container
			it.stop

			put_string(", remove at end")
			int_list.remove

			put_nl

			put_string(" ")
			print_collection
			put_string(" ")
			reverse_print
		end

	update_start is
			-- Change start of list.
		local
			it : P_ITERATOR[INTEGER]
		do
			check valid_list: not int_list.is_empty end
			
			it := int_list.iterator
			
			put_string("replace start: ")
			goto_start(it)
			int_list.replace(200)
			print_collection

			put_string("insert start: ")
			goto_start(it)
			int_list.insert(201)
			print_collection

			put_string("remove start: ")
			goto_start(it)
			int_list.remove
			print_collection
		end

	update_end is
			-- Change end of list.
		local
			it : P_ITERATOR[INTEGER]
		do
			check valid_list: not int_list.is_empty end
			
			it := int_list.iterator
			
			put_string("replace end: ")
			goto_end(it)
			int_list.replace(300)
			print_collection

			put_string("insert end: ")
			goto_end(it)
			int_list.insert(301)
			print_collection

			put_string("remove end: ")
			goto_end(it)
			int_list.remove
			print_collection
		end
	
	simple_forward_move(qty : INTEGER) is
			-- Partial iteration.
		require
			valid_size: qty > 0
		local
			i : INTEGER;
			it : P_ITERATOR[INTEGER]
		do
			from
				i := 1
				it := int_list.iterator
				it.first
			invariant
				not_empty: not int_list.is_empty
				inside_limits: (i <= qty) implies (not it.outside)
			variant
				qty - i + 1
			until
				i > qty
			loop
				it.synchronize_container
				it.forth
				i := i + 1
			end
			if not it.outside then
				it.stop
			end
		end

	sort_list is
			-- Sort driver.
		local
			cr : P_ORDER_COMPARABLE[INTEGER]
		do
			!!cr
			int_list.set_order(cr)
			int_list.sort

			put_string("sorted: ")
			print_collection
		end

	print_collection is
			-- Complete forward iteration and content access.
		local
			it : P_ITERATOR[INTEGER]
		do
			put_string("forward iteration: (")
			put_string(int_list.count.out)
			put_string(") ")

			from
				it := int_list.iterator
				it.first
			invariant
				first_ok: not int_list.is_empty
			variant
				it.forward_variant
			until
				it.outside
			loop
				put_string(it.item.out)
				put_string(" ")
				it.forth
			end
			put_nl
		end
	
	reverse_print is
			-- Complete reverse iteration.
		local
			it : P_ITERATOR[INTEGER]
		do
			put_string("backward iteration: ")
			from
				it := int_list.iterator
				it.last
			invariant
				first_ok: not int_list.is_empty
			variant
				it.backward_variant
			until
				it.outside
			loop
				put_string(it.item.out)
				put_string(" ")
				it.back
			end
			put_nl
		end
	
end
