-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Test
	description: "Toy string compression tool"
	toy: "Compression quite slow. Decompression OK. Not very efficient (20-30 per cent on text)"
	interface: mixin

class TEST_COMPRESS

feature { NONE }
	
	Escape_character : CHARACTER is '%/27/' -- must be outside [min, max]
	Min_character_code : INTEGER is 32
	Max_character_code : INTEGER is 255

	base : INTEGER is
		once
			Result := Max_character_code - Min_character_code + 1
		end

	min_base : INTEGER is 5 -- minimal lenght worth encoding

feature
	
	compress(in : STRING) : STRING is
		local
			substrings : P_CATALOG[INTEGER,TC_SUBSTRING]
			it : P_ITERATOR[INTEGER]
			sub : TC_SUBSTRING
			i, compressed_i : INTEGER
			maxf,maxfind,maxfind_pos : INTEGER
		do
			!!substrings.make
			!!Result.make(0)

			from
				i := 1
				compressed_i := 1
			invariant
				race: compressed_i >= i
			variant
				in.count + 1 - i 
			until i > in.count loop

				-- add recursively all minimal strings
				-- in hash table.
				if i > min_base then
					!!sub.make(in,i-min_base,min_base)
					substrings.add(sub.offset,sub)
				end

				-- now look ahead to find possible compression
				if compressed_i = i then
					
					-- compress
					sub := Void
					if (i > min_base) and (i + min_base - 1 <= in.count) then
						-- check if there is at least one minimal solution
						!!sub.make(in,i,min_base)
						substrings.key_search(sub)
						if substrings.found then
							
							-- find best
							from
								it := substrings.item_list.iterator
								it.first
								maxfind := 0
							variant it.forward_variant until it.outside loop
								maxf := match(in,it.item,i)
								if maxf > maxfind then
									maxfind := maxf
									maxfind_pos := it.item
								end
								it.forth
							end
							
							-- has at least one, search better
							check has_some: maxfind > 0 end
							!!sub.make(in,maxfind_pos,maxfind)
						else
							-- not
							sub := Void
						end
					end
					
					if (sub /= Void) and then (sub.offset // base < base) then --? overflow?
						check 
							no_offset_overflow: sub.offset // base < base 
							no_lenght_overflow: sub.lenght < base
						end

						Result.append_character(Escape_character)
						Result.append_character(int_to_char(sub.lenght))
						Result.append_character(int_to_char(sub.offset // base))
						Result.append_character(int_to_char(sub.offset \\ base))

						compressed_i := compressed_i + sub.lenght
					else -- not found

						if in.item(i) = Escape_character then
							Result.append_character(Escape_character) -- twice
						end
						Result.append_character(in.item(i))
						compressed_i := compressed_i + 1
					end
				end
				i := i + 1
			end
		ensure
			is_copy: Result /= Void
			done: decompress(Result).is_equal(in)
		end

	decompress(in : STRING) : STRING is
			-- Decompress a stream of characters. A characters is copied 'as is' 
			-- unless it is Escape_character which is followed by [lenght] (one 
			-- character) and [offset] (2 characters).
			-- The output string gets a copy of substring(offset,lenght) of the 
			-- already decompressed output.
		local
			i, encoded : INTEGER
			offset, len : INTEGER
			ch : CHARACTER
		do
			!!Result.make(0)
			from
				i := 1
			variant in.count + 1 - i until i > in.count loop
				ch := in.item(i)

				inspect
					encoded
				when 0 then
					if ch = Escape_character then
						encoded := 1
						check reset: offset = 0 and len = 0 end
					else
						Result.append_character(ch)
					end
				when 1 then
					if ch = Escape_character then
						Result.append_character(Escape_character)
						encoded := 0
					else
						len := char_to_int(ch)
						encoded := encoded + 1
					end
				when 2 then
					offset := base * char_to_int(ch)
					encoded := encoded + 1
				when 3 then
					offset := offset + char_to_int(ch)
					Result.append_string(Result.substring(offset,offset+len-1))
					encoded := 0
					offset := 0 len := 0
				end
				
				i := i + 1
			end
		end

feature { NONE }
	
	match(in : STRING; from1,from2 : INTEGER) : INTEGER is
			-- How many characters are common starting from from1/from2?
		require
			valid_str: in /= Void
			from1_ok: from1 > 0 and from1 <= in.count
			from2_ok: from2 > 0 and from2 <= in.count
			in_order: from1 < from2
		local
			cnt : INTEGER
		do
			-- find maximum
			from
				cnt := in.count
			variant
				base - Result
			until
				((from1+Result >= from2) or (from2+Result > cnt) or (Result+1 >= base)) -- bounds
					or else (in.item(from1+Result) /= in.item(from2+Result)) -- actual comparison
			loop
				Result := Result + 1
			end
		end

feature { NONE } -- Encode numbers
	
	char_to_int(ch : CHARACTER) : INTEGER is
		do
			Result := ch.code - Min_character_code
		ensure
			bound: Result >= 0 and Result < base
			done: int_to_char(Result) = ch
		end

	int_to_char(int : INTEGER) : CHARACTER is
			-- Int to char in stream.
		require
			bound: int >= 0 and int < base
		local
			sys : P_SYSTEM
		do
			!!sys
			Result := sys.code_to_char(int + Min_character_code)
		ensure
			done: char_to_int(Result) = int
		end
	
end -- class TEST_COMPRESS
