-- Pylon: a foundation library; version 0.81(beta); 1997-05-08

-- 
-- Copyright (c) 1997 AlterSoft, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY ALTERSOFT AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL ALTERSOFT OR THE AUTHORS BE LIABLE
-- TO ANY PARTY FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE
-- CODE.
-- 

indexing
	cluster: Pylon, Test
	description: "Test tree"
	interface: client

class TEST_TREE

inherit
	TESTER
		end

creation
	execute

feature
	
	execute is
		do
			put_title("GENERAL TREE:")
			init_tree
			put_title("preorder iteration:")
			display_tree(dtree.preorder_iterator)
			put_title("levelorder iteration:")
			display_tree(dtree.levelorder_iterator)
			put_title("postorder iteration:")
			display_tree(dtree.postorder_iterator)

			put_title("subtree:")
			test_subtree
		end

	lock : P_ONEWAY_ITERATOR[STRING] is
		do
			Result := dtree.preorder_iterator
			Result.synchronize
			Result.lock 
		end

	unlock(it : P_ONEWAY_ITERATOR[STRING]) is
		do
			it.unlock
			it.synchronize_container
			it.stop
		end

	init_tree is
		local
			tok, tok2 : P_ONEWAY_ITERATOR[STRING]
		do
			!!dtree.make
			
			dtree.add_root("p_linked_list")
			
				dtree.add_child("p_list")
				tok := lock -- push position
				
					dtree.add_child("p_traversable")
					tok2 := lock -- push
					
						dtree.add_child("p_oneway_traversable")
							dtree.add_child("p_container")
						
					unlock(tok2) -- pop
					dtree.add_right("p_searchable")
					tok2 := lock -- push
					
						dtree.add_child("p_container")

					unlock(tok2) -- pop
					dtree.add_left("p_sortable")
						dtree.add_child("p_container")

				unlock(tok) -- pop
				dtree.add_right("p_node_container")
				dtree.add_child("p_lockable")
		ensure
			no_locked: not dtree.has_locked
		end

	test_subtree is
		do
			dtree.root
			dtree := dtree.subtree
			display_tree(dtree.preorder_iterator)
		end

	display_tree(it : P_ONEWAY_ITERATOR[STRING]) is
		local
			outstr : STRING; 
			i : INTEGER
		do
			from
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				!!outstr.make(0)
				it.synchronize_container

				-- indent
				from i := dtree.depth variant i until i = 0 loop
					outstr.append_string("  ")
					i := i - 1
				end

				-- append path
				outstr.append_string(it.item)
				
				-- print
				put_title(outstr)
					
				it.forth
			end
		end

feature { NONE }

	dtree : P_GENERAL_TREE[STRING] 

end
